@@ -62,11 +62,7 @@ static guint timeout_id = 0;
 static gboolean
 kill_connection_manager (gpointer data)
 {
-#ifdef ENABLE_DEBUG
-  if (!_tp_debug_flag_is_set (TP_DEBUG_PERSIST) && !connections_exist)
-#else
-  if (!connections_exist)
-#endif
+  if (!_TP_DEBUG_IS_PERSISTENT && !connections_exist)
     {
       g_debug ("no connections, and timed out");
       g_object_unref (manager);
@@ -237,6 +233,7 @@ tp_run_connection_manager (const char *prog_name,
   timeout_id = g_timeout_add (DIE_TIME, kill_connection_manager, NULL);
 
   g_main_loop_run (mainloop);
+  g_main_loop_unref (mainloop);
 
   return 0;
 }
